--desc: core DBA utility to get space and help diagnose runaway logs/tran logs
SELECT dbz.log_reuse_wait_desc,dbz.* FROM sys.databases dbz WHERE (DB_NAME() = 'master' OR dbz.name = DB_NAME())

execute xp_fixeddrives
 SELECT 
    DISTINCT(volume_mount_point), 
    total_bytes/1048576 AS Size_in_MB, 
    (total_bytes/1048576) - (available_bytes/1048576) AS Used_in_MB,
    available_bytes/1048576 AS Free_in_MB,
        (total_bytes/1048576) / 1024 AS Size_in_GB, 
    (((total_bytes - available_bytes)/1048576) /1024) AS Used_in_GB,
    (available_bytes/1048576) / 1024 AS Free_in_GB,
    CONVERT(Decimal(5,2),(((((total_bytes - available_bytes)/1048576) /1024))/((total_bytes/1048576) / 1024 * 1.0)) * 100) AS PercentUsed,
    CONVERT(Decimal(5,2),(((available_bytes/1048576) / 1024)/((total_bytes/1048576) / 1024 * 1.0)) * 100) AS PercentFree
  FROM sys.master_files AS f 
  CROSS APPLY sys.dm_os_volume_stats(f.database_id, f.FILE_ID)
  GROUP BY 
    volume_mount_point, 
    total_bytes,
    
    available_bytes 
  ORDER BY 1;


DECLARE @db sysname =  CASE WHEN DB_NAME() = 'master' THEN NULL ELSE db_name() end
SET NOCOUNT ON
DECLARE @CurrentDB NVARCHAR(128)

-- The table #tblServerDatabases holds the names of databases on the server.
-- This table is used here to allow us to loop through each database,
-- rather than to run undocumented procedures, such as sp_msforeachdb
-- (which eliminated the need for a loop).
IF OBJECT_ID('tempdb..#tblServerDatabases', 'U') IS NOT NULL        
        DROP TABLE #tblServerDatabases

CREATE TABLE #tblServerDatabases (DBName NVARCHAR(128),state_desc NVARCHAR(128),recovery_model_desc NVARCHAR(128))

-- The table #tblDBFilesExtendedInfo holds the data and log files info
-- (name, size, used, free spaces, etc.).
IF OBJECT_ID('tempdb..#tblDBFilesExtendedInfo', 'U') IS NOT NULL
        DROP TABLE #tblDBFilesExtendedInfo

CREATE TABLE #tblDBFilesExtendedInfo (
        Idx INT IDENTITY(1, 1),
        FileID INT,
        FileGroupID INT,
        TotalExtents BIGINT,
        UsedExtents BIGINT,
        DBFileName NVARCHAR(128),
        recovery_model_desc NVARCHAR(128),
        LogicalFileName NVARCHAR(128),
        DBFilePath NVARCHAR(1024),
        DBFileType VARCHAR(16),
        DBName NVARCHAR(128),
        [TotalFileSize(MB)] MONEY,
        [TotalUsed(MB)] MONEY,
        [TotalFree(MB)] MONEY,
        [SpaceUsed(%)] MONEY,
        Status INT)

-- This table will hold the output of sp_helpfile, for each database.
-- This is needed in order to get the log file path and file name.
IF OBJECT_ID('tempdb..#tblDBFilesBasicInfo', 'U') IS NOT NULL
        DROP TABLE #tblDBFilesBasicInfo

CREATE TABLE #tblDBFilesBasicInfo (
        DBName NVARCHAR(128),
        DBFileName NVARCHAR(128),
        FileID INT,
        FilePath NVARCHAR(1024),
        FileGroupDesc NVARCHAR(128),
        FileSizeKB NVARCHAR(64),
        MaxSizeDesc NVARCHAR(64),
        Growth NVARCHAR(64),
        Usage NVARCHAR(64))


-- First - the data files are handled throught the
-- DBCC SHOWFILESTATS command.

INSERT INTO #tblServerDatabases (DBName,state_desc,recovery_model_desc)
SELECT dbz.name,dbz.state_desc,dbz.recovery_model_desc
FROM master.sys.databases  dbz


SELECT @CurrentDB = MIN(DBName)
FROM #tblServerDatabases
WHERE state_desc='ONLINE'

WHILE @CurrentDB IS NOT NULL
BEGIN
        INSERT INTO #tblDBFilesExtendedInfo(
                FileID,
                FileGroupID,
                TotalExtents,
                UsedExtents, 
                DBFileName,
                DBFilePath)


        EXEC ('USE [' + @CurrentDB + '] 
SELECT 
	f.file_id,
	fg.data_space_id, 
	f.size / 8 AS TotalExtents,
	CAST(FILEPROPERTY(f.name, ''SpaceUsed'')  / 8 AS bigint) AS UsedExtents,
	f.name AS Name,
	f.physical_name AS FileName
FROM sys.database_files AS f WITH (NOLOCK)
INNER JOIN sys.data_spaces AS fg WITH (NOLOCK)
ON f.data_space_id = fg.data_space_id OPTION (RECOMPILE);') --USE [NDD_ADP_RAW] DBCC SHOWFILESTATS'

        UPDATE #tblDBFilesExtendedInfo
        SET    DBName = @CurrentDB,
               DBFileType = 'Data File' 
        WHERE DBName IS NULL

        -- Run the sp_helpfile in order to get log file data.
        INSERT INTO #tblDBFilesBasicInfo(
                DBFileName,
                FileID,
                FilePath,
                FileGroupDesc,
                FileSizeKB,
                MaxSizeDesc,
                Growth,
                Usage)
        EXEC ('USE [' + @CurrentDB + '] EXEC sp_helpfile ')

        UPDATE #tblDBFilesBasicInfo
        SET    DBName = @CurrentDB
        WHERE DBName IS NULL

        SELECT @CurrentDB = MIN(DBName)
        FROM #tblServerDatabases WITH (NOLOCK)
        WHERE DBName > @CurrentDB
        AND state_desc='ONLINE'
END

-- Update the total file size, used and free space, based on the
-- extents information returned from DBCC SHOWFILESTATS.
UPDATE #tblDBFilesExtendedInfo
SET     [DBFileName] = RIGHT(DBFilePath, CHARINDEX('\', REVERSE(DBFilePath)) -1),
        [TotalFileSize(MB)] = CAST(((TotalExtents*64) / 1024.00) AS MONEY),
        [TotalUsed(MB)] = CAST(((UsedExtents*64) / 1024.00) AS MONEY),
        [TotalFree(MB)] = CAST(((TotalExtents*64) / 1024.00) AS MONEY)
                          - CAST(((UsedExtents*64) / 1024.00) AS MONEY),
        [SpaceUsed(%)] = CASE
                        WHEN CAST(((TotalExtents*64) / 1024.00) AS MONEY) = 0.0 THEN 0.0
                        ELSE (CAST(((UsedExtents*64) / 1024.00) AS MONEY)*100)
                              / CAST(((TotalExtents*64) / 1024.00) AS MONEY)
                        END

-- We are now done with the data file statuses, and we shall move
-- on to get the log files info, by using DBCC SQLPERF(LOGSPACE)
INSERT INTO #tblDBFilesExtendedInfo (DBName, [TotalFileSize(MB)], [SpaceUsed(%)], Status)
EXEC('DBCC SQLPERF(LOGSPACE)')

UPDATE a
SET     a.[TotalUsed(MB)] = (a.[SpaceUsed(%)]/100.00)*a.[TotalFileSize(MB)],
        a.[TotalFree(MB)] = (1.0 - (a.[SpaceUsed(%)]/100.00))*a.[TotalFileSize(MB)],
        a.DBFileType = 'Log file',
        a.DBFilePath = b.FilePath,
        a.DBFileName = RIGHT(b.FilePath, CHARINDEX('\', REVERSE(b.FilePath)) -1)
FROM #tblDBFilesExtendedInfo a
        INNER JOIN #tblDBFilesBasicInfo b
        ON a.DBName = b.DBName
WHERE a.DBFileType IS NULL
        AND b.Usage = 'log only'

--we want the logical file name as well, in case we need it for other purposes, like getting everything in a single spot to move files
UPDATE MyTarget
SET MyTarget.LogicalFileName = mff.name
FROM #tblDBFilesExtendedInfo MyTarget
INNER JOIN  sys.master_files  mff
ON MyTarget.DBFilePath = mff.physical_name

UPDATE MyTarget
SET MyTarget.recovery_model_desc = dbz.recovery_model_desc
FROM #tblDBFilesExtendedInfo MyTarget
INNER JOIN  sys.databases  dbz ON MyTarget.DBName = dbz.name

SET NOCOUNT OFF
SELECT * FROM #tblServerDatabases WHERE DBName NOT IN(SELECT DBName FROM #tblDBFilesExtendedInfo)
AND (DBName = @db OR @db IS NULL)
-- That's it. We now need to return a readable recordset.
SELECT  '--Round1 use these commands only if tempdb will not shrink,and you HAVE to shrink it:' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'CHECKPOINT; -- first chkpt' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'CHECKPOINT; --yes, two chkpts in a row then try to shrink' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  '--Round2 if checkpoint did not help then this is the next step' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FREEPROCCACHE;' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FREESYSTEMCACHE (''ALL'');' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  '--Round3 if checkpoint did not help then this is the next step' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FREEPROCCACHE;' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FREESYSTEMCACHE (''ALL'');' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FLUSHPROCINDB (2) --tempdb' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC FREESESSIONCACHE;' WHERE DB_NAME() = 'tempdb' UNION ALL
SELECT  'DBCC DROPCLEANBUFFERS;' WHERE DB_NAME() = 'tempdb' 
SELECT  
        CONVERT(VARCHAR(128),@@serverName) AS ServerName,
        DBName,
        [recovery_model_desc],
        DBFileType,
        LogicalFileName,
        DBFileName,
        DBFilePath,
        [TotalFileSize(MB)],
        [TotalUsed(MB)],
        [SpaceUsed(%)],
        [TotalFree(MB)],
        CASE 
        WHEN [TotalFileSize(MB)] -10000 < 0 THEN '--unable to shrink below 10GB
        USE ' + quotename(DBName) + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10) + 'DBCC SHRINKFILE (N''' + LogicalFileName + ''' , ' + CONVERT(VARCHAR,CONVERT(INT,([TotalFileSize(MB)]))) + ')  -- was ' + CONVERT(VARCHAR,CONVERT(INT,[TotalFileSize(MB)])) + ' --smallest Possible is '+ CONVERT(VARCHAR,[TotalUsed(MB)])
        ELSE 
        'USE ' + quotename(DBName) + CHAR(13) + CHAR(10) + 'GO' + CHAR(13) + CHAR(10) + 'DBCC SHRINKFILE (N''' + LogicalFileName + ''' , ' + CONVERT(VARCHAR,CONVERT(INT,([TotalFileSize(MB)] -10000))) + ')  -- was ' + CONVERT(VARCHAR,CONVERT(INT,[TotalFileSize(MB)])) + ' --smallest Possible is '+ CONVERT(VARCHAR,[TotalUsed(MB)]) --DBCC SHRINKFILE (N'MirthConnect_EAI_log' , 472879)
        END   AS [ShrinkBy10GBcmd]
FROM #tblDBFilesExtendedInfo WITH (NOLOCK)
WHERE (DBName = @db OR @db IS NULL)
UNION ALL
SELECT 
        CONVERT(VARCHAR(128),@@serverName) AS ServerName,
    DBName, recovery_model_desc,state_desc,state_desc,state_desc,state_desc,NULL,NULL,NULL,NULL,NULL 
FROM #tblServerDatabases WHERE DBName NOT IN(SELECT DBName FROM #tblDBFilesExtendedInfo)
AND ( DBName = @db OR @db IS NULL)
ORDER BY [TotalFree(MB)] DESC



SELECT  
        CONVERT(VARCHAR(128),@@serverName) AS ServerName,
        DBName,
        [recovery_model_desc],
        SUM([TotalFileSize(MB)]) AS [TotalFileSize(MB)],
        SUM([TotalUsed(MB)]) AS [TotalUsed(MB)],
        (SUM([TotalUsed(MB)]) / SUM([TotalFileSize(MB)]) * 100) AS [SpaceUsed(%)],
        SUM([TotalFree(MB)]) AS [TotalFree(MB)]
FROM #tblDBFilesExtendedInfo WITH (NOLOCK)
GROUP BY 
        DBName,
        [recovery_model_desc]